linux-kernel.vger.kernel.org archive mirror
 help / color / mirror / Atom feed
From: "Mirela Rabulea (OSS)" <mirela.rabulea@oss.nxp.com>
To: mchehab@kernel.org, hverkuil-cisco@xs4all.nl,
	shawnguo@kernel.org, robh+dt@kernel.org
Cc: paul.kocialkowski@bootlin.com, linux-media@vger.kernel.org,
	linux-kernel@vger.kernel.org, linux-imx@nxp.com,
	s.hauer@pengutronix.de, aisheng.dong@nxp.com,
	daniel.baluta@nxp.com, robert.chiras@nxp.com,
	laurentiu.palcu@nxp.com, mark.rutland@arm.com,
	devicetree@vger.kernel.org, p.zabel@pengutronix.de,
	ezequiel@collabora.com,
	laurent.pinchart+renesas@ideasonboard.com,
	niklas.soderlund+renesas@ragnatech.se,
	dafna.hirschfeld@collabora.com,
	Mirela Rabulea <mirela.rabulea@nxp.com>
Subject: [PATCH v4 08/11] media: Add parsing for APP14 data segment in jpeg helpers
Date: Mon,  2 Nov 2020 05:08:18 +0200	[thread overview]
Message-ID: <20201102030821.3049-9-mirela.rabulea@oss.nxp.com> (raw)
In-Reply-To: <20201102030821.3049-1-mirela.rabulea@oss.nxp.com>

From: Mirela Rabulea <mirela.rabulea@nxp.com>

According to Rec. ITU-T T.872 (06/2012) 6.5.3
APP14 segment is for color encoding, it contains a transform flag, which
may have values of 0, 1 and 2 and are interpreted as follows:
0 - CMYK for images that are encoded with four components
  - RGB for images that are encoded with three components
1 - An image encoded with three components using YCbCr colour encoding.
2 - An image encoded with four components using YCCK colour encoding.

This is used in imx-jpeg decoder, to distinguish between
YUV444 and RGB24.

Signed-off-by: Mirela Rabulea <mirela.rabulea@nxp.com>
---
 drivers/media/v4l2-core/v4l2-jpeg.c | 39 +++++++++++++++++++++++++++--
 include/media/v4l2-jpeg.h           | 16 ++++++++++--
 2 files changed, 51 insertions(+), 4 deletions(-)

diff --git a/drivers/media/v4l2-core/v4l2-jpeg.c b/drivers/media/v4l2-core/v4l2-jpeg.c
index 8947fd95c6f1..a8198f98c158 100644
--- a/drivers/media/v4l2-core/v4l2-jpeg.c
+++ b/drivers/media/v4l2-core/v4l2-jpeg.c
@@ -45,6 +45,7 @@ MODULE_LICENSE("GPL");
 #define DHP	0xffde	/* hierarchical progression */
 #define EXP	0xffdf	/* expand reference */
 #define APP0	0xffe0	/* application data */
+#define APP14	0xffee	/* application data for colour encoding */
 #define APP15	0xffef
 #define JPG0	0xfff0	/* extensions */
 #define JPG13	0xfffd
@@ -444,8 +445,35 @@ static int jpeg_skip_segment(struct jpeg_stream *stream)
 	return jpeg_skip(stream, len - 2);
 }
 
+/* Rec. ITU-T T.872 (06/2012) 6.5.3 */
+static int jpeg_parse_app14_data(struct jpeg_stream *stream,
+				 struct v4l2_jpeg_app14_data *app14)
+{
+	int Lp;
+	int skip;
+	int tf;
+
+	if (!app14)
+		return 0; /* do not parse */
+
+	Lp = jpeg_get_word_be(stream);
+	if (Lp < 0)
+		return Lp;
+	app14->len = Lp;
+
+	/* get to Ap12 */
+	jpeg_skip(stream, 11);
+	tf = jpeg_get_byte(stream);
+	if (tf < 0)
+		return tf;
+	app14->transform_flag = tf;
+
+	skip = Lp - 2 - 11;
+	return jpeg_skip(stream, skip);
+}
+
 /**
  * jpeg_parse_header - locate marker segments and optionally parse headers
  * @buf: address of the JPEG buffer, should start with a SOI marker
  * @len: length of the JPEG buffer
  * @out: returns marker segment positions and optionally parsed headers
@@ -476,6 +504,10 @@ int v4l2_jpeg_parse_header(void *buf, size_t len, struct v4l2_jpeg_header *out)
 	if (marker != SOI)
 		return -EINVAL;
 
+	/* init value to signal if this marker is not present */
+	if (out->app14)
+		out->app14->len = -1;
+
 	/* loop through marker segments */
 	while ((marker = jpeg_next_marker(&stream)) >= 0) {
 		switch (marker) {
@@ -519,7 +551,10 @@ int v4l2_jpeg_parse_header(void *buf, size_t len, struct v4l2_jpeg_header *out)
 			ret = jpeg_parse_restart_interval(&stream,
 							&out->restart_interval);
 			break;
-
+		case APP14:
+			ret = jpeg_parse_app14_data(&stream,
+						    out->app14);
+			break;
 		case SOS:
 			ret = jpeg_reference_segment(&stream, &out->sos);
 			if (ret < 0)
@@ -444,8 +445,35 @@ static int jpeg_skip_segment(struct jpeg_stream *stream)
 /**
- * jpeg_parse_header - locate marker segments and optionally parse headers
+ * v4l2_jpeg_parse_header - locate marker segments and optionally parse headers
  * @buf: address of the JPEG buffer, should start with a SOI marker
  * @len: length of the JPEG buffer
diff --git a/include/media/v4l2-jpeg.h b/include/media/v4l2-jpeg.h
index ddba2a56c321..2b02d89f20b4 100644
--- a/include/media/v4l2-jpeg.h
+++ b/include/media/v4l2-jpeg.h
@@ -87,6 +87,16 @@ struct v4l2_jpeg_scan_header {
 	/* Ss, Se, Ah, and Al are not used by any driver */
 };
 
+/**
+ * struct v4l2_jpeg_app14_data - APP14 data segment
+ * @len: Lp application data segment length, becomes -1 if APP14 is not present
+ * @transform_flag: Ap12 byte for color encoding
+ */
+struct v4l2_jpeg_app14_data {
+	u16 len;
+	u8 transform_flag;
+};
+
 /**
  * struct v4l2_jpeg_header - parsed JPEG header
  * @sof: pointer to frame header and size
@@ -98,12 +108,13 @@ struct v4l2_jpeg_scan_header {
  * @quantization_tables: references to four quantization tables, optional
  * @huffman_tables: references to four Huffman tables in DC0, DC1, AC0, AC1
  *                  order, optional
+ * @app14: pointer to app14 data, optional
  * @restart_interval: number of MCU per restart interval, Ri
  * @ecs_offset: buffer offset in bytes to the entropy coded segment
  *
  * When this structure is passed to v4l2_jpeg_parse_header, the optional scan,
  * quantization_tables, and huffman_tables pointers must be initialized to NULL
  * or point at valid memory.
  */
 struct v4l2_jpeg_header {
 	struct v4l2_jpeg_reference sof;
@@ -117,6 +128,7 @@ struct v4l2_jpeg_header {
 	struct v4l2_jpeg_scan_header *scan;
 	struct v4l2_jpeg_reference *quantization_tables;
 	struct v4l2_jpeg_reference *huffman_tables;
+	struct v4l2_jpeg_app14_data *app14;
 	u16 restart_interval;
 	size_t ecs_offset;
 };
@@ -98,12 +108,13 @@ struct v4l2_jpeg_scan_header {
  *
  * When this structure is passed to v4l2_jpeg_parse_header, the optional scan,
- * quantization_tables, and huffman_tables pointers must be initialized to NULL
- * or point at valid memory.
+ * quantization_tables, huffman_tables and app14 pointers must be initialized
+ * to NULL or point at valid memory.
  */
 struct v4l2_jpeg_header {
-- 
2.17.1


  parent reply	other threads:[~2020-11-02  3:10 UTC|newest]

Thread overview: 23+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2020-11-02  3:08 [PATCH v4 00/11] Add V4L2 driver for i.MX8 JPEG Encoder/Decoder Mirela Rabulea (OSS)
2020-11-02  3:08 ` [PATCH v4 01/11] media: v4l: Add packed YUV444 24bpp pixel format Mirela Rabulea (OSS)
2020-11-02  3:08 ` [PATCH v4 02/11] firmware: imx: scu-pd: Add power domains for imx-jpeg Mirela Rabulea (OSS)
2020-11-02  3:08 ` [PATCH v4 03/11] media: dt-bindings: Add bindings for i.MX8QXP/QM JPEG driver Mirela Rabulea (OSS)
2020-11-04 22:38   ` Rob Herring
2020-11-09 21:59     ` [EXT] " Mirela Rabulea
2020-11-02  3:08 ` [PATCH v4 04/11] media: imx-jpeg: Add V4L2 driver for i.MX8 JPEG Encoder/Decoder Mirela Rabulea (OSS)
2020-11-02 16:20   ` Laurentiu Palcu
2020-11-06  1:05     ` Mirela Rabulea
2020-11-04 16:03   ` kernel test robot
2020-11-02  3:08 ` [PATCH v4 05/11] arm64: dts: imx8qxp: Add jpeg encoder/decoder nodes Mirela Rabulea (OSS)
2020-11-02  3:08 ` [PATCH v4 06/11] Add maintainer for IMX jpeg v4l2 driver Mirela Rabulea (OSS)
2020-11-02 16:25   ` Laurentiu Palcu
2020-11-02  3:08 ` [PATCH v4 07/11] media: imx-jpeg: Fix v4l2-compliance streaming tests on decoder Mirela Rabulea (OSS)
2020-11-02  3:08 ` Mirela Rabulea (OSS) [this message]
2020-11-02  3:08 ` [PATCH v4 09/11] media: Quit parsing stream if doesn't start with SOI Mirela Rabulea (OSS)
2020-11-02  3:08 ` [PATCH v4 10/11] media: Avoid parsing quantization and huffman tables Mirela Rabulea (OSS)
2020-11-02  3:08 ` [PATCH v4 11/11] media: imx-jpeg: Use v4l2 jpeg helpers in mxc-jpeg Mirela Rabulea (OSS)
2020-11-04 11:52 ` [PATCH v4 00/11] Add V4L2 driver for i.MX8 JPEG Encoder/Decoder Hans Verkuil
2020-11-04 11:55   ` Hans Verkuil
2020-11-04 13:27     ` [EXT] " Mirela Rabulea
2020-11-04 14:08       ` Hans Verkuil
2020-11-06  1:04         ` Mirela Rabulea

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20201102030821.3049-9-mirela.rabulea@oss.nxp.com \
    --to=mirela.rabulea@oss.nxp.com \
    --cc=aisheng.dong@nxp.com \
    --cc=dafna.hirschfeld@collabora.com \
    --cc=daniel.baluta@nxp.com \
    --cc=devicetree@vger.kernel.org \
    --cc=ezequiel@collabora.com \
    --cc=hverkuil-cisco@xs4all.nl \
    --cc=laurent.pinchart+renesas@ideasonboard.com \
    --cc=laurentiu.palcu@nxp.com \
    --cc=linux-imx@nxp.com \
    --cc=linux-kernel@vger.kernel.org \
    --cc=linux-media@vger.kernel.org \
    --cc=mark.rutland@arm.com \
    --cc=mchehab@kernel.org \
    --cc=mirela.rabulea@nxp.com \
    --cc=niklas.soderlund+renesas@ragnatech.se \
    --cc=p.zabel@pengutronix.de \
    --cc=paul.kocialkowski@bootlin.com \
    --cc=robert.chiras@nxp.com \
    --cc=robh+dt@kernel.org \
    --cc=s.hauer@pengutronix.de \
    --cc=shawnguo@kernel.org \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for NNTP newsgroup(s).